/******************************************************************************
*                              Steffen Hofmann                                *
*                                 Hamburg                                     *
*                               mc_sho@gmx.de                                 *
*******************************************************************************
*  $Source: $
*-----------------------------------------------------------------------------*
*  Datum der 1. Version  :  08.07.2003
*  Geraete Bezeichnung   :  MEGA-128-BOARD
*-----------------------------------------------------------------------------*
*- $Id: $
*-----------------------------------------------------------------------------*
*
*       Headerfile
*       ----------
*       mcp2515.h
*       V1.00
******************************************************************************/
#ifndef __MCP2515_H
#define __MCP2515_H

#include "types.h"
/*
Version     Date        Description
----------------------------------------------------------------------
v1.00       2003/12/11  Initial release

*/

// Define MCP2515 register addresses

#define MCP_RXF0SIDH    0x00   // FILTER 0 STANDARD IDENTIFIER HIGH
#define MCP_RXF0SIDL    0x01   // FILTER 0 STANDARD IDENTIFIER LOW
#define MCP_RXF0EID8    0x02   // FILTER 0 EXTENDED IDENTIFIER HIGH
#define MCP_RXF0EID0    0x03   // FILTER 0 EXTENDED IDENTIFIER LOW
#define MCP_RXF1SIDH    0x04   // FILTER 1 STANDARD IDENTIFIER HIGH
#define MCP_RXF1SIDL    0x05   // FILTER 1 STANDARD IDENTIFIER LOW
#define MCP_RXF1EID8    0x06   // FILTER 1 EXTENDED IDENTIFIER HIGH
#define MCP_RXF1EID0    0x07   // FILTER 1 EXTENDED IDENTIFIER LOW
#define MCP_RXF2SIDH    0x08   // FILTER 2 STANDARD IDENTIFIER HIGH
#define MCP_RXF2SIDL    0x09   // FILTER 2 STANDARD IDENTIFIER LOW
#define MCP_RXF2EID8    0x0A   // FILTER 2 EXTENDED IDENTIFIER HIGH
#define MCP_RXF2EID0    0x0B   // FILTER 2 EXTENDED IDENTIFIER LOW
#define MCP_BFPCTRL     0x0C   // RXnBF PIN CONTROL AND STATUS
#define MCP_TXRTSCTRL   0x0D   // 
#define MCP_CANSTAT     0x0E   // CANSTAT - CAN STATUS REGISTER
#define MCP_CANCTRL     0x0F   // CANCTRL - CAN CONTROL REGISTER
#define MCP_RXF3SIDH    0x10   // FILTER 3 STANDARD IDENTIFIER HIGH
#define MCP_RXF3SIDL    0x11   // FILTER 3 STANDARD IDENTIFIER LOW
#define MCP_RXF3EID8    0x12   // FILTER 3 EXTENDED IDENTIFIER HIGH
#define MCP_RXF3EID0    0x13   // FILTER 3 EXTENDED IDENTIFIER LOW
#define MCP_RXF4SIDH    0x14   // FILTER 4 STANDARD IDENTIFIER HIGH
#define MCP_RXF4SIDL    0x15   // FILTER 4 STANDARD IDENTIFIER LOW
#define MCP_RXF4EID8    0x16   // FILTER 4 EXTENDED IDENTIFIER HIGH
#define MCP_RXF4EID0    0x17   // FILTER 4 EXTENDED IDENTIFIER LOW
#define MCP_RXF5SIDH    0x18   // FILTER 5 STANDARD IDENTIFIER HIGH
#define MCP_RXF5SIDL    0x19   // FILTER 5 STANDARD IDENTIFIER LOW
#define MCP_RXF5EID8    0x1A   // FILTER 5 EXTENDED IDENTIFIER HIGH
#define MCP_RXF5EID0    0x1B   // FILTER 5 EXTENDED IDENTIFIER LOW
#define MCP_TEC         0x1C   // TEC - TRANSMIT ERROR COUNTER
#define MCP_REC         0x1D   // REC - RECEIVER ERROR COUNTER
#define MCP_RXM0SIDH    0x20   // MASK 0 STANDARD IDENTIFIER HIGH
#define MCP_RXM0SIDL    0x21   // MASK 0 STANDARD IDENTIFIER LOW
#define MCP_RXM0EID8    0x22   // MASK 0 EXTENDED IDENTIFIER HIGH
#define MCP_RXM0EID0    0x23   // MASK 0 EXTENDED IDENTIFIER LOW
#define MCP_RXM1SIDH    0x24   // MASK 1 STANDARD IDENTIFIER HIGH
#define MCP_RXM1SIDL    0x25   // MASK 1 STANDARD IDENTIFIER LOW
#define MCP_RXM1EID8    0x26   // MASK 1 EXTENDED IDENTIFIER HIGH
#define MCP_RXM1EID0    0x27   // MASK 1 EXTENDED IDENTIFIER LOW

#define MCP_CNF3        0x28   // CNF3 - CONFIGURATION 1 (ADDRESS: 28h)
#define MCP_CNF2        0x29   // CNF2 - CONFIGURATION 1 (ADDRESS: 29h)
#define MCP_CNF1        0x2A   // CNF1 - CONFIGURATION 1 (ADDRESS: 2Ah)

#define MCP_CANINTE     0x2B   // CANINTE - INTERRUPT ENABLE
#define MCP_CANINTF     0x2C   // CANINTF - INTERRUPT FLAG
#define MCP_EFLG        0x2D   // EFLG - ERROR FLAG

#define MCP_TXB0CTRL    0x30   // TRANSMIT BUFFER 0 CONTROL REGISTER
#define MCP_TXB0SIDH    0x31   // TRANSMIT BUFFER 0 STANDARD IDENTIFIER HIGH
#define MCP_TXB0SIDL    0x32   // TRANSMIT BUFFER 0 STANDARD IDENTIFIER LOW
#define MCP_TXB0EID8    0x33   // TRANSMIT BUFFER 0 EXTENDED IDENTIFIER HIGH
#define MCP_TXB0EID0    0x34   // TRANSMIT BUFFER 0 EXTENDED IDENTIFIER LOW
#define MCP_TXB0DLC     0x35   // TRANSMIT BUFFER 0 DATA LENGTH CODE
#define MCP_TXB0D0      0x36   // TRANSMIT BUFFER 0 DATA BYTE 0
#define MCP_TXB0D1      0x37   // TRANSMIT BUFFER 0 DATA BYTE 1
#define MCP_TXB0D2      0x38   // TRANSMIT BUFFER 0 DATA BYTE 2
#define MCP_TXB0D3      0x39   // TRANSMIT BUFFER 0 DATA BYTE 3
#define MCP_TXB0D4      0x3A   // TRANSMIT BUFFER 0 DATA BYTE 4
#define MCP_TXB0D5      0x3B   // TRANSMIT BUFFER 0 DATA BYTE 5
#define MCP_TXB0D6      0x3C   // TRANSMIT BUFFER 0 DATA BYTE 6
#define MCP_TXB0D7      0x3D   // TRANSMIT BUFFER 0 DATA BYTE 7

#define MCP_TXB1CTRL    0x40   // TRANSMIT BUFFER 1 CONTROL REGISTER
#define MCP_TXB1SIDH    0x41   // TRANSMIT BUFFER 1 STANDARD IDENTIFIER HIGH
#define MCP_TXB1SIDL    0x42   // TRANSMIT BUFFER 1 STANDARD IDENTIFIER LOW
#define MCP_TXB1EID8    0x43   // TRANSMIT BUFFER 1 EXTENDED IDENTIFIER HIGH
#define MCP_TXB1EID0    0x44   // TRANSMIT BUFFER 1 EXTENDED IDENTIFIER LOW
#define MCP_TXB1DLC     0x45   // TRANSMIT BUFFER 1 DATA LENGTH CODE
#define MCP_TXB1D0      0x46   // TRANSMIT BUFFER 1 DATA BYTE 0
#define MCP_TXB1D1      0x47   // TRANSMIT BUFFER 1 DATA BYTE 1
#define MCP_TXB1D2      0x48   // TRANSMIT BUFFER 1 DATA BYTE 2
#define MCP_TXB1D3      0x49   // TRANSMIT BUFFER 1 DATA BYTE 3
#define MCP_TXB1D4      0x4A   // TRANSMIT BUFFER 1 DATA BYTE 4
#define MCP_TXB1D5      0x4B   // TRANSMIT BUFFER 1 DATA BYTE 5
#define MCP_TXB1D6      0x4C   // TRANSMIT BUFFER 1 DATA BYTE 6
#define MCP_TXB1D7      0x4D   // TRANSMIT BUFFER 1 DATA BYTE 7

#define MCP_TXB2CTRL    0x50   // TRANSMIT BUFFER 2 CONTROL REGISTER
#define MCP_TXB2SIDH    0x51   // TRANSMIT BUFFER 2 STANDARD IDENTIFIER HIGH
#define MCP_TXB2SIDL    0x52   // TRANSMIT BUFFER 2 STANDARD IDENTIFIER LOW
#define MCP_TXB2EID8    0x53   // TRANSMIT BUFFER 2 EXTENDED IDENTIFIER HIGH
#define MCP_TXB2EID0    0x54   // TRANSMIT BUFFER 2 EXTENDED IDENTIFIER LOW
#define MCP_TXB2DLC     0x55   // TRANSMIT BUFFER 2 DATA LENGTH CODE
#define MCP_TXB2D0      0x56   // TRANSMIT BUFFER 2 DATA BYTE 0
#define MCP_TXB2D1      0x57   // TRANSMIT BUFFER 2 DATA BYTE 1
#define MCP_TXB2D2      0x58   // TRANSMIT BUFFER 2 DATA BYTE 2
#define MCP_TXB2D3      0x59   // TRANSMIT BUFFER 2 DATA BYTE 3
#define MCP_TXB2D4      0x5A   // TRANSMIT BUFFER 2 DATA BYTE 4
#define MCP_TXB2D5      0x5B   // TRANSMIT BUFFER 2 DATA BYTE 5
#define MCP_TXB2D6      0x5C   // TRANSMIT BUFFER 2 DATA BYTE 6
#define MCP_TXB2D7      0x5D   // TRANSMIT BUFFER 2 DATA BYTE 7

#define MCP_RXB0CTRL    0x60   // RECEIVE BUFFER 0 CONTROL
#define MCP_RXB0SIDH    0x61   // RECEIVE BUFFER 0 STANDARD IDENTIFIER HIGH < 10:3 >
#define MCP_RXB0SIDL    0x62   // RECEIVE BUFFER 0 STANDARD IDENTIFIER LOW
#define MCP_RXB0EID8    0x63   // RECEIVE BUFFER 0 EXTENDED IDENTIFIER HIGH
#define MCP_RXB0EID0    0x64   // RECEIVE BUFFER 0 EXTENDED IDENTIFIER LOW
#define MCP_RXB0DLC     0x65   // RECEIVE BUFFER 0 DATA LENGHT CODE
#define MCP_RXB0D0      0x66   // RECEIVE BUFFER 0 DATA BYTE 0
#define MCP_RXB0D1      0x67   // RECEIVE BUFFER 0 DATA BYTE 1
#define MCP_RXB0D2      0x68   // RECEIVE BUFFER 0 DATA BYTE 2
#define MCP_RXB0D3      0x69   // RECEIVE BUFFER 0 DATA BYTE 3
#define MCP_RXB0D4      0x6A   // RECEIVE BUFFER 0 DATA BYTE 4
#define MCP_RXB0D5      0x6B   // RECEIVE BUFFER 0 DATA BYTE 5
#define MCP_RXB0D6      0x6C   // RECEIVE BUFFER 0 DATA BYTE 6
#define MCP_RXB0D7      0x6D   // RECEIVE BUFFER 0 DATA BYTE 7

#define MCP_RXB1CTRL    0x70   // RECEIVE BUFFER 1 CONTROL
#define MCP_RXB1SIDH    0x71   // RECEIVE BUFFER 1 STANDARD IDENTIFIER HIGH
#define MCP_RXB1SIDL    0x72   // RECEIVE BUFFER 1 STANDARD IDENTIFIER LOW
#define MCP_RXB1EID8    0x73   // RECEIVE BUFFER 1 EXTENDED IDENTIFIER HIGH
#define MCP_RXB1EID0    0x74   // RECEIVE BUFFER 1 EXTENDED IDENTIFIER LOW
#define MCP_RXB1DLC     0x75   // RECEIVE BUFFER 1 DATA LENGHT CODE
#define MCP_RXB1D0      0x76   // RECEIVE BUFFER 1 DATA BYTE 0
#define MCP_RXB1D1      0x77   // RECEIVE BUFFER 1 DATA BYTE 1
#define MCP_RXB1D2      0x78   // RECEIVE BUFFER 1 DATA BYTE 2
#define MCP_RXB1D3      0x79   // RECEIVE BUFFER 1 DATA BYTE 3
#define MCP_RXB1D4      0x7A   // RECEIVE BUFFER 1 DATA BYTE 4
#define MCP_RXB1D5      0x7B   // RECEIVE BUFFER 1 DATA BYTE 5
#define MCP_RXB1D6      0x7C   // RECEIVE BUFFER 1 DATA BYTE 6
#define MCP_RXB1D7      0x7D   // RECEIVE BUFFER 1 DATA BYTE 7

#define SIDH            0
#define SIDL            1
#define EID8            2
#define EID0            3

#define MCP_TX_INT      0x1C   // Enable all transmit interrupts
#define MCP_TX01_INT    0x0C   // Enable TXB0 and TXB1 interrupts
#define MCP_RX_INT      0x03   // Enable receive interrupts
#define MCP_NO_INT      0x00   // Disable all interrupts

#define MCP_TX01_MASK   0x14
#define MCP_TX_MASK     0x54

// Define SPI Instruction Set

#define MCP_WRITE       0x02

#define MCP_READ        0x03

#define MCP_BITMOD      0x05

#define MCP_LOAD_TX0    0x40
#define MCP_LOAD_TX1    0x42
#define MCP_LOAD_TX2    0x44

#define MCP_RTS_TX0     0x81
#define MCP_RTS_TX1     0x82
#define MCP_RTS_TX2     0x84
#define MCP_RTS_ALL     0x87

#define MCP_READ_RX0    0x90
#define MCP_READ_RX1    0x94

#define MCP_READ_STATUS 0xA0

#define MCP_RX_STATUS   0xB0

#define MCP_RESET       0xC0
//-----------------------------------------------------------------------------
// TRANSMIT BUFFER CTRL
//-----------------------------------------------------------------------------
#define TXBnCTRL_ABTF   BIT(6)       //- Message Aborted Flag
                                     // 1 = Message was aborted
                                     // 0 = Message completed transmission successfully
#define TXBnCTRL_MLOA   BIT(5)       //- Message Lost Arbitration
                                     // 1 = Message lost arbitration while being sent
                                     // 0 = Message did not lose arbitration while being sent
#define TXBnCTRL_TXERR  BIT(4)       //- Transmission Error Detected
                                     // 1 = A bus error occurred while the message was being transmitted
                                     // 0 = No bus error occurred while the message was being transmitted
#define TXBnCTRL_TXREQ  BIT(3)       //- Message Transmit Request
                                     // 1 = Buffer is currently pending transmission
                                     //    (MCU sets this bit to request message be transmitted - 
                                     //     bit is automatically cleared when the message is sent)  
                                     // 0 = Buffer is not currently pending transmission
                                     //     (MCU can clear this bit to request a message abort)
#define TXBnCTRL_MASK          0x03  // 
#define TXBnCTRL_HIGH_PRIO     0x03  // Highest Message Priority
#define TXBnCTRL_HIGH_IMM_PRIO 0x02  // High Intermediate Message Priority
#define TXBnCTRL_LOW_IMM_PRIO  0x01  // Low Intermediate Message Priority ( Fehler im Datenblt. 0x03 ? )
#define TXBnCTRL_LOW_PRIO      0x00  // Lowest Message Priority
//-----------------------------------------------------------------------------
// TXnRTS PIN CONTROL AND STATUS REGISTER Bit define
//-----------------------------------------------------------------------------
#define TXRTSCTRL_B2RTS        BIT(5) // TX2RTS Pin State
#define TXRTSCTRL_B1RTS        BIT(4) // TX1RTS Pin State
#define TXRTSCTRL_B0RTS        BIT(3) // TX0RTS Pin State
#define TXRTSCTRL_B2RTSM       BIT(2) // TX2RTS Pin mode   0 = Digital input
#define TXRTSCTRL_B1RTSM       BIT(1) // TX1RTS Pin mode   0 = Digital input
#define TXRTSCTRL_B0RTSM       BIT(0) // TX0RTS Pin mode   0 = Digital input
//-----------------------------------------------------------------------------
// RECEIVE BUFFER 0 CONTROL
//-----------------------------------------------------------------------------
#define RXB0CTRL_RXM_OFF      0x60    // Turn mask/filters off; receive any message
#define RXB0CTRL_RXM_EXT      0x40    // Receive only valid messages with extended identifiers that meet filter criteria
#define RXB0CTRL_RXM_STD      0x20    // Receive only valid messages with standard identifiers that meet filter criteria
#define RXB0CTRL_RXM_ALL      0x00    // Receive all valid messages using either standard or extended identifiers that meet filter
                                      // criteria
#define RXB0CTRL_RXM_MASK     0x60    // Bit Mask
#define RXB0CTRL_RXRTR        BIT(3)  // Received Remote Transfer Request ;0 = No Remote Transfer Request Received
#define RXB0CTRL_BUKT         BIT(2)  // Rollover Enable  ;0 = Rollover disabled
#define RXB0CTRL_BUKT1        BIT(1)  // Read Only Copy of BUKT Bit (used internally by the MCP2515)
#define RXB0CTRL_FILHIT       BIT(0)  // Filter Hit - indicates which acceptance filter enabled reception of message
                                      //  1 = Acceptance Filter 1 (RXF1)
                                      //  0 = Acceptance Filter 0 (RXF0)
//-----------------------------------------------------------------------------
// RECEIVE BUFFER 1 CONTROL
//-----------------------------------------------------------------------------
#define RXB1CTRL_RXM_OFF      0x60    // Turn mask/filters off; receive any message
#define RXB1CTRL_RXM_EXT      0x40    // Receive only valid messages with extended identifiers that meet filter criteria
#define RXB1CTRL_RXM_STD      0x20    // Receive only valid messages with standard identifiers that meet filter criteria
#define RXB1CTRL_RXM_ALL      0x00    // Receive all valid messages using either standard or extended identifiers that meet filter
                                      // criteria
#define RXB1CTRL_RXM_MASK     0x60    // Bit Mask
#define RXB1CTRL_RXRTR        BIT(3)  // Received Remote Transfer Request ;0 = No Remote Transfer Request Received
#define RXB1CTRL_FILHIT_RXF5  0x05    // Acceptance Filter 5 (RXF5)
#define RXB1CTRL_FILHIT_RXF4  0x04    // Acceptance Filter 4 (RXF4)
#define RXB1CTRL_FILHIT_RXF3  0x03    // Acceptance Filter 3 (RXF3)
#define RXB1CTRL_FILHIT_RXF2  0x02    // Acceptance Filter 2 (RXF2)
#define RXB1CTRL_FILHIT_RXF1  0x01    // Acceptance Filter 1 (RXF1) (Only if BUKT bit set in RXB0CTRL)
#define RXB1CTRL_FILHIT_RXF0  0x00    // Acceptance Filter 0 (RXF0) (Only if BUKT bit set in RXB0CTRL)

//-----------------------------------------------------------------------------
// RXnBF PIN CONTROL AND STATUS
//-----------------------------------------------------------------------------
#define RXnBF_B1BFS           BIT(5)  // RX1BF Pin State (digital output mode only)
#define RXnBF_B0BFS           BIT(4)  // RX0BF Pin State (digital output mode only)
#define RXnBF_B1BFE           BIT(3)  // RX1BF Pin Function Enable
#define RXnBF_B0BFE           BIT(2)  // RX0BF Pin Function Enable
#define RXnBF_B1BFM           BIT(1)  // RX1BF Pin Operation Mode ;0 = Digital output mode
#define RXnBF_B0BFM           BIT(0)  // RX0BF Pin Operation Mode ;0 = Digital output mode

//-----------------------------------------------------------------------------
// ERROR FLAG Bit defines
//-----------------------------------------------------------------------------
#define EFLG_RX1OVR           BIT(7)  // Receive Buffer 1 Overflow Flag
#define EFLG_RX0OVR           BIT(6)  // Receive Buffer 0 Overflow Flag
#define EFLG_TXBO             BIT(5)  // Bus-Off Error Flag
#define EFLG_TXEP             BIT(4)  // Transmit Error-Passive Flag
#define EFLG_RXEP             BIT(3)  // Receive Error-Passive Flag
#define EFLG_TXWAR            BIT(2)  // Transmit Error Warning Flag
#define EFLG_RXWAR            BIT(1)  // Receive Error Warning Flag
#define EFLG_EWARN            BIT(0)  // Error Warning Flag

//-----------------------------------------------------------------------------
// CANINTE - INTERRUPT ENABLE
//-----------------------------------------------------------------------------
#define CANINTE_MERRE         BIT(7)  // Message Error Interrupt Enable
#define CANINTE_WAKIE         BIT(6)  // Wakeup Interrupt Enable
#define CANINTE_ERRIE         BIT(5)  // Error Interrupt Enable (multiple sources in EFLG register)
#define CANINTE_TX2IE         BIT(4)  // Transmit Buffer 2 Empty Interrupt Enable
#define CANINTE_TX1IE         BIT(3)  // Transmit Buffer 1 Empty Interrupt Enable
#define CANINTE_TX0IE         BIT(2)  // Transmit Buffer 0 Empty Interrupt Enable
#define CANINTE_RX1IE         BIT(1)  // Receive Buffer 1 Full Interrupt Enable
#define CANINTE_RX0IE         BIT(0)  // Receive Buffer 0 Full Interrupt Enable
#define CANINTE_ALL_DISABLE   0x00    // Disable all Interrupts

//-----------------------------------------------------------------------------
// CANINTF Register Bits
//-----------------------------------------------------------------------------
#define CANINTF_RX0IF       0x01
#define CANINTF_RX1IF       0x02
#define CANINTF_TX0IF       0x04
#define CANINTF_TX1IF       0x08
#define CANINTF_TX2IF       0x10
#define CANINTF_ERRIF       0x20
#define CANINTF_WAKIF       0x40
#define CANINTF_MERRF       0x80
#define CANINTF_ALL_DISABLE 0x00

//-----------------------------------------------------------------------------
// TXBnSIDL - TRANSMIT BUFFER n STANDARD IDENTIFIER LOW
//-----------------------------------------------------------------------------
#define TXBnSIDL_EXIDE      BIT(3)    // Extended Identifier Enable


// CANCTRL Register Values

#define MODE_NORMAL     0x00
#define MODE_SLEEP      0x20
#define MODE_LOOPBACK   0x40
#define MODE_LISTENONLY 0x60
#define MODE_CONFIG     0x80
#define MODE_POWERUP    0xE0
#define MODE_MASK       0xE0
#define ABORT_TX        0x10
#define MODE_ONESHOT    0x08
#define CLKOUT_ENABLE   0x04
#define CLKOUT_DISABLE  0x00
#define CLKOUT_PS1      0x00
#define CLKOUT_PS2      0x01
#define CLKOUT_PS4      0x02
#define CLKOUT_PS8      0x03

// CANSTAT Register Values

#define CANSTAT_MASK    0x0E   
#define STAT_NOINT      0x00   // 000 = No Interrupt
#define STAT_ERRORINT   0x02   // 001 = Error Interrupt
#define STAT_WAKEUP     0x04   // 010 = Wake Up Interrupt
#define STAT_TXB0_INT   0x06   // 011 = TXB0 Interrupt
#define STAT_TXB1_INT   0x08   // 100 = TXB1 Interrupt
#define STAT_TXB2_INT   0x0A   // 101 = TXB2 Interrupt
#define STAT_RXB0_INT   0x0C   // 110 = RXB0 Interrupt
#define STAT_RXB1_INT   0x0E   // 111 = RXB1 Interrupt

// CNF1 Register Values

#define SJW1            0x00
#define SJW2            0x40
#define SJW3            0x80
#define SJW4            0xC0


// CNF2 Register Values

#define BTLMODE         0x80
#define SAMPLE_1X       0x00
#define SAMPLE_3X       0x40


// CNF3 Register Values

#define SOF_ENABLE      0x80
#define SOF_DISABLE     0x00
#define WAKFIL_ENABLE   0x40
#define WAKFIL_DISABLE  0x00

//-----------------------------------------------------------------------------
// BitTiming 
//-----------------------------------------------------------------------------
// 1000Kbps bei 16MHz
#define SPEED_1000_CNF1 0x00
#define SPEED_1000_CNF2 0x90
#define SPEED_1000_CNF3 0x02

// 1000Kbps bei 20MHz
//#define SPEED_1000_CNF1 0x00
//#define SPEED_1000_CNF2 0xA0
//#define SPEED_1000_CNF3 0x02

// 500Kbps bei 16MHz
#define SPEED_500_CNF1  0x00
#define SPEED_500_CNF2  0xB8
#define SPEED_500_CNF3  0x05

// 500Kbps bei 40MHz
//#define SPEED_500_CNF1  0x01
//#define SPEED_500_CNF2  0xBA
//#define SPEED_500_CNF3  0x07

// 250Kbps bei 16MHz
#define SPEED_250_CNF1  0x01
#define SPEED_250_CNF2  0xB8
#define SPEED_250_CNF3  0x05

// 125Kbps bei 16MHz
#define SPEED_125_CNF1  0x03
#define SPEED_125_CNF2  0xB8
#define SPEED_125_CNF3  0x05

// 50Kbps bei 16MHz
#define SPEED_50_CNF1  0x03
#define SPEED_50_CNF2  0xBA
#define SPEED_50_CNF3  0x07
//-----------------------------------------------------------------------------

// CANINTF Register Bits

#define MCP_RX0IF       0x01
#define MCP_RX1IF       0x02
#define MCP_TX0IF       0x04
#define MCP_TX1IF       0x08
#define MCP_TX2IF       0x10
#define MCP_ERRIF       0x20
#define MCP_WAKIF       0x40
#define MCP_MERRF       0x80

#define CLEAR_ALL_INTERRUPS     0x00

#define CAN_EXT_ID 1
#define CAN_NO_EXT_ID 0


//-----------------------------------------------------------------------------
//- Funktionen
//-----------------------------------------------------------------------------
void CAN_reset(void);
BYTE CAN_State (void);
void CAN_Init (DWORD ID,BYTE CNF1_data,BYTE CNF2_data,BYTE CNF3_data);
void CAN_Sleep ( void );
void CAN_Wake_up ( void );
void CAN_Tx0 ( DWORD CAN_ID, BYTE EXT_ID, BYTE DLC, BYTE *DTA);
void CAN_Tx1 ( DWORD CAN_ID, BYTE EXT_ID, BYTE DLC, BYTE *DTA);
void CAN_Tx2 ( DWORD CAN_ID, BYTE EXT_ID, BYTE DLC, BYTE *DTA);
void CAN_Rx0 ( DWORD *CAN_ID, BYTE *EXT_ID, BYTE *DLC, BYTE *DTA);
void CAN_Rx1 ( DWORD *CAN_ID, BYTE *EXT_ID, BYTE *DLC, BYTE *DTA);
void CAN_Read_Can_ID( BYTE mcp_addr, BYTE* ext, DWORD* can_id );
void CAN_Write_Can_ID( BYTE mcp_addr, BYTE ext, DWORD can_id );
BYTE CAN_Chk (void);
void CAN_IntClear (void);
void CAN_ClearRX0 (void);
void CAN_ClearRX1 (void);
#endif
