/******************************************************************************
*                              Steffen Hofmann                                *
*                                 Hamburg                                     *
*                               mc_sho@gmx.de                                 *
*******************************************************************************
*  $Source: $
*-----------------------------------------------------------------------------*
*  Datum der 1. Version  :  08.07.2003
*  Geraete Bezeichnung   :  MEGA-128-BOARD
*-----------------------------------------------------------------------------*
*- $Id: $
*-----------------------------------------------------------------------------*
*
*       Source
*       ----------
*       mcp2515.c
*       V1.00
******************************************************************************/

#include "hw.h"
#include "mcp2515.h"
#include "spi.h"
#include "types.h"

#ifdef PR_RTX
#include "task.h"
#include "pr_rtx_macros_defs.h"
#else
#include "timer.h"
#endif

//-----------------------------------------------------------------------------
//- extern
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
//- locale functionen
//-----------------------------------------------------------------------------
void SPI_mcp_select (void);
void SPI_mcp_unselect (void);
void CAN_ModifyRegister ( BYTE regaddr, BYTE mask, BYTE regvalue);
void CAN_Reset( void );
BYTE CAN_ReadRegister( BYTE regaddr );
void CAN_ReadManyRegisters( BYTE regaddr, BYTE len, BYTE *dta );
void CAN_WriteRegister( BYTE regaddr, BYTE regvalue );
void CAN_WriteManyRegisters( BYTE regaddr, BYTE len, BYTE *dta );
void CAN_Mode (BYTE Mode);

static BYTE RettConf;

//-----------------------------------------------------------------------------
// function:	ModifyRegister
// description:	Modify one MCP2515 register.
//-----------------------------------------------------------------------------
void CAN_ModifyRegister ( BYTE regaddr, BYTE mask, BYTE regvalue)
{

   SPI_mcp_select();	// Select the MCP2515
   SpiWriteByte( MCP_BITMOD );
   SpiWriteByte( regaddr );
   SpiWriteByte( mask );
   SpiWriteByte( regvalue );
   SPI_mcp_unselect();	// Deselect the MCP2515

#ifdef PR_RTX
   PR_Wait_Ticks( 10 );		// short delay
#else
   
   wait_ms( 1 );
#endif
}

//-----------------------------------------------------------------------------
// function:	State
// description:	Reads state of MCP2515 over SPI.
//-----------------------------------------------------------------------------
BYTE CAN_State (void)
{
   BYTE rreg;
   
   SPI_mcp_select();	            // Select the MCP2515
   SpiWriteByte( MCP_READ_STATUS );	 
   rreg = SpiReadByte();	        // Read back the register
   SPI_mcp_unselect();	          // Deselect the MCP2515

#ifdef PR_RTX
   PR_Wait_Ticks( 10 );		        // short delay
#else
   
   wait_ms( 1 );
#endif
   
   return rreg;
}

//-----------------------------------------------------------------------------
// function:	ResetMCP2515
// description:	Reset the MCP2515.
//-----------------------------------------------------------------------------
void CAN_Reset( void )
{
#ifdef CAN_EXT_RESET

   CBI( RES_PORT, RES_CAN);  //- RES_CAN low
	  

#ifdef PR_RTX
   PR_Wait_Ticks( 10 );		              // short delay
#else
   
   wait_ms( 250 );
#endif
   
   SBI( RES_PORT, RES_CAN);  //- RES_CAN high
#endif	  
}

//-----------------------------------------------------------------------------
// function:	ReadRegister
// description:	Read 1 register from the MCP2515 over SPI.
//-----------------------------------------------------------------------------
BYTE CAN_ReadRegister( BYTE regaddr )
{
   BYTE rreg;
   
   SPI_mcp_select();	        // Select the MCP2515
   SpiWriteByte( MCP_READ );	// Send the Read command
   SpiWriteByte( regaddr );	// Send the register
   rreg = SpiReadByte();	// Read back the register
   SPI_mcp_unselect();	        // Deselect the MCP2515

#ifdef PR_RTX
   PR_Wait_Ticks( 10 );		    // short delay
#else
   
   wait_ms( 1 );
#endif

   return rreg;
}

//-----------------------------------------------------------------------------
// function:	ReadManyRegister
// description:	Read multiple registers from the
//		MCP2515 over SPI.
//-----------------------------------------------------------------------------
void CAN_ReadManyRegisters( BYTE regaddr, BYTE len, BYTE *dta )
{

   BYTE x;
   
   SPI_mcp_select();         // Select the MCP2515
   SpiWriteByte( MCP_READ );	// Send the Read command
   SpiWriteByte( regaddr );	 // Send the register
   for( x=0; x<len; x++ )
   {
      *dta = SpiReadByte( );
       dta++;
   }
   SPI_mcp_unselect();  	    // Deselect the MCP2515

#ifdef PR_RTX
   PR_Wait_Ticks( 10 );		    // short delay
#else
   
   wait_ms( 1 );
#endif
}

//-----------------------------------------------------------------------------
// function:	WriteRegister
// description:	Write 1 MCP2515 register.
//-----------------------------------------------------------------------------
void CAN_WriteRegister( BYTE regaddr, BYTE regvalue )
{

   SPI_mcp_select();              // Select the MCP2515
   SpiWriteByte( MCP_WRITE );
   SpiWriteByte( regaddr );
   SpiWriteByte( regvalue );
   SPI_mcp_unselect();            // Deselect the MCP2515

#ifdef PR_RTX
   PR_Wait_Ticks( 10 );		    // short delay
#else
   
   wait_ms( 1 );
#endif
}

//-----------------------------------------------------------------------------
// function:	WriteManyRegisters
// description:	Write multiple MCP2515 registers.
//-----------------------------------------------------------------------------
void CAN_WriteManyRegisters( BYTE regaddr, BYTE len, BYTE *dta )
{

   BYTE x;
   
   SPI_mcp_select(); ;	// Select the MCP2515
   SpiWriteByte( MCP_WRITE );	// Send the Read command
   SpiWriteByte( regaddr );	// Send the register
   for( x=0; x<len; x++ )
   {
      SpiWriteByte( *dta );
      dta++;
   }
   SPI_mcp_unselect();	// Deselect the MCP2515

#ifdef PR_RTX
   PR_Wait_Ticks( 10 );		    // short delay
#else
   
   wait_ms( 1 );
#endif
}

//-----------------------------------------------------------------------------
// function:	Switching between the MCP Modes
// description:	
//-----------------------------------------------------------------------------
void CAN_Mode (BYTE Mode)
{
   CAN_WriteRegister (MCP_CANCTRL,Mode);
}

//-----------------------------------------------------------------------------
// function:	Initalization of the MCP2515 (Normal Operation Mode)
// description:	
//-----------------------------------------------------------------------------
void CAN_Init (DWORD ID,BYTE CNF1_data,BYTE CNF2_data,BYTE CNF3_data)
{
  WORD idl,idh;
  
  //- Reset MCP2515
  CAN_Reset ();
  //- MCP2515 in Config Mode
  CAN_Mode (MODE_CONFIG);              
  //- Baudrate
  CAN_WriteRegister (MCP_CNF1,CNF1_data);  //CNF1  Configuration Registers..
  CAN_WriteRegister (MCP_CNF2,CNF2_data);  //CNF2
  CAN_WriteRegister (MCP_CNF3,( CNF3_data | SOF_DISABLE | WAKFIL_DISABLE ));  //CNF3
  //- drei TX als Eingaenge
  CAN_WriteRegister (MCP_TXRTSCTRL,(TXRTSCTRL_B2RTSM | TXRTSCTRL_B1RTSM | TXRTSCTRL_B0RTSM)); 

  CAN_WriteRegister ( MCP_RXB0CTRL, RXB0CTRL_RXM_OFF );
  CAN_WriteRegister ( MCP_RXB1CTRL, RXB1CTRL_RXM_OFF );

    // Mark all filter bits as don't care:
    CAN_Write_Can_ID(MCP_RXM0SIDH, CAN_EXT_ID, 0);
    CAN_Write_Can_ID(MCP_RXM1SIDH, CAN_EXT_ID, 0);
    // Anyway, set all filters to 0:
    CAN_Write_Can_ID(RXB1CTRL_FILHIT_RXF0, CAN_NO_EXT_ID, 0);
    CAN_Write_Can_ID(RXB1CTRL_FILHIT_RXF1, CAN_NO_EXT_ID, 0);
    CAN_Write_Can_ID(RXB1CTRL_FILHIT_RXF2, CAN_NO_EXT_ID, 0);
    CAN_Write_Can_ID(RXB1CTRL_FILHIT_RXF3, CAN_NO_EXT_ID, 0);
    CAN_Write_Can_ID(RXB1CTRL_FILHIT_RXF4, CAN_NO_EXT_ID, 0);
    CAN_Write_Can_ID(RXB1CTRL_FILHIT_RXF5, CAN_NO_EXT_ID, 0);
    
/*  MCP_WriteRegister (RXM0SIDH,0x03);         //MASK  unterste 5 Bits auswerten
  MCP_WriteRegister (RXM0SIDL,0xE0);         //MASK
  MCP_WriteRegister (RXM1SIDH,0x03);         //MASK 
  MCP_WriteRegister (RXM1SIDL,0xE0);         //MASK

  MCP_WriteRegister (RXF0SIDH,0x00);         // Recieve Filters
  MCP_WriteRegister (RXF0SIDL,0x00);         //....
  idh = ((ID & 0x07F8) >> 3);
  idl = ((ID & 0x0007) << 5);
  MCP_WriteRegister (RXF1SIDH,idh); //
  MCP_WriteRegister (RXF1SIDL,idl); //
  MCP_WriteRegister (RXF2SIDH,0x00);         // alle anderen nur NOTAUS
  MCP_WriteRegister (RXF2SIDL,0x00);
  MCP_WriteRegister (RXF3SIDH,0x00);
  MCP_WriteRegister (RXF3SIDL,0x00);
  MCP_WriteRegister (RXF4SIDH,0x00);
  MCP_WriteRegister (RXF4SIDL,0x00);
  MCP_WriteRegister (RXF5SIDH,0x00);
  MCP_WriteRegister (RXF5SIDL,0x00);*/

  //CAN_WriteRegister (MCP_CANINTE, CANINTE_RX1IE + CANINTE_RX0IE ); // Enable Receive interrups
  CAN_WriteRegister (MCP_CANINTE,CANINTF_ALL_DISABLE ); 
  CAN_WriteRegister (MCP_BFPCTRL,(RXnBF_B0BFM |  // RX0 not Ints
                                  RXnBF_B1BFM |  // RX1 not Ints
                                  RXnBF_B0BFE |
                                  RXnBF_B1BFE ));
                                  
                                  
  CAN_Mode(MODE_NORMAL);              // Normal Mode

}

//-----------------------------------------------------------------------------
// function:	Goto Sleep Mode
// description:	
//-----------------------------------------------------------------------------
void CAN_Sleep ( void )
{
  CAN_WriteRegister(MCP_CANINTE, CANINTE_WAKIE ); 
  CAN_Mode(MODE_SLEEP);
}

//-----------------------------------------------------------------------------
// function:	Wake up from Sleep Mode
// description:	Must be done after WakeUp Interupt from the MCP
//              when the MCP is in Sleep Mode
//-----------------------------------------------------------------------------
void CAN_Wake_up ( void )
{
   CAN_WriteRegister(MCP_CANINTF, CANINTF_ALL_DISABLE );
}

//-----------------------------------------------------------------------------
// function:	Transmit Message  TX0
// description:	
//-----------------------------------------------------------------------------
void CAN_Tx0 ( DWORD CAN_ID, BYTE EXT_ID, BYTE DLC, BYTE *DTA)
{
  //MCP_WriteRegister (TXB0SIDL,CAN_ID_LOW); //Std-IDentifyer low
  //MCP_WriteRegister (TXB0SIDH,CAN_ID_HIGH); //Std-IDentifyer hi
  
  if( DLC > 8 )
     return;
     
  CAN_Write_Can_ID( MCP_TXB0SIDH, EXT_ID, CAN_ID );
  CAN_WriteRegister (MCP_TXB0DLC,DLC); //Data length
  CAN_WriteManyRegisters (MCP_TXB0D0,DLC,DTA);
  CAN_WriteRegister (MCP_TXB0CTRL,0b00001011); // Tx-Ctr
//  do

//  {

//    notsend = (0x08 & MCP_ReadRegister(TXB0CTRL));

//  } 

//  while (notsend == 0x00);

}
//-----------------------------------------------------------------------------
// function:	Transmit Message  TX1
// description:	
//-----------------------------------------------------------------------------
void CAN_Tx1 ( DWORD CAN_ID, BYTE EXT_ID, BYTE DLC, BYTE *DTA)
{
  //MCP_WriteRegister (TXB0SIDL,CAN_ID_LOW); //Std-IDentifyer low
  //MCP_WriteRegister (TXB0SIDH,CAN_ID_HIGH); //Std-IDentifyer hi
  
  CAN_Write_Can_ID( MCP_TXB1SIDH, EXT_ID, CAN_ID );
  CAN_WriteRegister (MCP_TXB1DLC,DLC); //Data length
  CAN_WriteManyRegisters (MCP_TXB1D0,DLC,DTA);
  CAN_WriteRegister (MCP_TXB1CTRL,0b00001011); // Tx-Ctr
}
//-----------------------------------------------------------------------------
// function:	Transmit Message  TX2
// description:	
//-----------------------------------------------------------------------------
void CAN_Tx2 ( DWORD CAN_ID, BYTE EXT_ID, BYTE DLC, BYTE *DTA)
{
  //MCP_WriteRegister (TXB0SIDL,CAN_ID_LOW); //Std-IDentifyer low
  //MCP_WriteRegister (TXB0SIDH,CAN_ID_HIGH); //Std-IDentifyer hi
  
  CAN_Write_Can_ID( MCP_TXB2SIDH, EXT_ID, CAN_ID );
  CAN_WriteRegister (MCP_TXB2DLC,DLC); //Data length
  CAN_WriteManyRegisters (MCP_TXB2D0,DLC,DTA);
  CAN_WriteRegister (MCP_TXB2CTRL,0b00001011); // Tx-Ctr
}    
//-----------------------------------------------------------------------------
// function:	Receive Message from Buffer 0
// description:	read Message and clear Flag
//-----------------------------------------------------------------------------
void CAN_Rx0 ( DWORD *CAN_ID, BYTE *EXT_ID, BYTE *DLC, BYTE *DTA)
{
   // Recieve from Rx0
   //WORD int_state;
   //WORD new_int_state;

  //*CAN_ID_HIGH = MCP_ReadRegister(RXB0SIDH);
  //*CAN_ID_LOW  = MCP_ReadRegister(RXB0SIDL);
  CAN_Read_Can_ID( MCP_RXB0SIDH, EXT_ID, CAN_ID ); 
  *DLC = CAN_ReadRegister(MCP_RXB0DLC);
  CAN_ReadManyRegisters(MCP_RXB0D0,*DLC,DTA);
}

//-----------------------------------------------------------------------------
// function:	Receive Message from Buffer 1
// description:	read Message and clear Flag
//-----------------------------------------------------------------------------
void CAN_Rx1 (DWORD *CAN_ID, BYTE *EXT_ID, BYTE *DLC, BYTE *DTA)
{
   // Recieve from Rx1
   //WORD int_state;
   //WORD new_int_state;

   //*CAN_ID_HIGH = MCP_ReadRegister(RXB1SIDH);
   //*CAN_ID_LOW  = MCP_ReadRegister(RXB1SIDL);
   CAN_Read_Can_ID( MCP_RXB1SIDH, EXT_ID, CAN_ID ); 
   *DLC = CAN_ReadRegister(MCP_RXB1DLC);
   CAN_ReadManyRegisters(MCP_RXB1D0,*DLC,DTA);
}
//-----------------------------------------------------------------------------
// function:	read CAN-ID std. or Ext
// description:	read can-id
//-----------------------------------------------------------------------------
void CAN_Read_Can_ID( BYTE mcp_addr, BYTE* ext, DWORD* can_id )
{
    BYTE tbufdata[4];
    
    *ext = 0;
    *can_id = 0;
    CAN_ReadManyRegisters( mcp_addr, 4, tbufdata );
    *can_id = (tbufdata[SIDH]<<3) + (tbufdata[SIDL]>>5);
    if ( (tbufdata[SIDL] & TXBnSIDL_EXIDE) ==  TXBnSIDL_EXIDE ) {
        *can_id = (*can_id<<2) + (tbufdata[SIDL] & 0x03);
        *can_id <<= 16;
        *can_id = *can_id +(tbufdata[EID8]<<8) + tbufdata[EID0];
        *ext = 1;
    }
}
//-----------------------------------------------------------------------------
// function:	write CAN-ID std. or Ext
// description:	write can-id
//-----------------------------------------------------------------------------
void CAN_Write_Can_ID( BYTE mcp_addr, BYTE ext, DWORD can_id )
{
    WORD canid;
    BYTE tbufdata[4];
    
    canid = (WORD)(can_id & 0x0FFFF);
    if ( ext == 1) {
        tbufdata[EID0] = (BYTE) (canid & 0xFF);
        tbufdata[EID8] = (BYTE) (canid / 256);
        canid = (WORD)( can_id / 0x10000L );
        tbufdata[SIDL] = (BYTE) (canid & 0x03);
        tbufdata[SIDL] += (BYTE) ((canid & 0x1C )*8);
        tbufdata[SIDL] |= TXBnSIDL_EXIDE;
        tbufdata[SIDH] = (BYTE) (canid / 32 );
    }
    else {
        tbufdata[SIDH] = (BYTE) (canid / 8 );
        tbufdata[SIDL] = (BYTE) ((canid & 0x07 )*32);
        tbufdata[EID0] = 0;
        tbufdata[EID8] = 0;
    }
    CAN_WriteManyRegisters( mcp_addr, 4, tbufdata );
}
//-----------------------------------------------------------------------------
// function:	Check Messages
// description:	is neq 0 if any RX-Buffer is not empty
//              returns 0x01 for Buffer0
//                      0x02 for Buffer1
//			0x03 for both Buffers
//-----------------------------------------------------------------------------
BYTE CAN_Chk (void)
{
  BYTE Buffer=0;
  Buffer = CAN_ReadRegister ( MCP_CANINTF );
  return (Buffer & 0x03);
}

//-----------------------------------------------------------------------------
// function:	Clear Interrupts 
// description:	
//-----------------------------------------------------------------------------
void CAN_IntClear (void)
{
   CAN_WriteRegister( MCP_CANINTF, CANINTF_ALL_DISABLE );
}

//-----------------------------------------------------------------------------
// function:	Clear Interrupt Flag RX0
// description:	
//-----------------------------------------------------------------------------
void CAN_ClearRX0 (void)
{
     CAN_ModifyRegister ( MCP_CANINTF, CANINTF_RX0IF, 0x00 );
//   unsigned int int_reg;

//   int_reg = MCP_ReadRegister(CANINTF);
//   int_reg = int_reg & 0xFE;
//   MCP_WriteRegister(CANINTF,int_reg);

}

//-----------------------------------------------------------------------------
// function:	Clear Interrupt Flag RX1
// description:	
//-----------------------------------------------------------------------------
void CAN_ClearRX1 (void)
{
     CAN_ModifyRegister ( MCP_CANINTF, CANINTF_RX1IF, 0x00 );
//   unsigned int int_reg;

//   int_reg = MCP_ReadRegister(CANINTF);
//   int_reg = int_reg & 0xFD;
//   MCP_WriteRegister(CANINTF,int_reg);

}

//-----------------------------------------------------------------------------
//- Local Functionen
//-----------------------------------------------------------------------------
//
//-----------------------------------------------------------------------------
// Selects the MCP2515.
// Note: mcpCS is set to zero also by SPI_mcp_RD_address(),
// SPI_mcp_WR_address() and SPI_mcp_RD_status().
//-----------------------------------------------------------------------------
void SPI_mcp_select (void)
{
   RettConf = SPCR;         // alte Config retten
   SPCR = SPI_MCP_WR;	    // neue SPI Config
    	
   CBI( CAN_PORT_CS, CS_CAN );    //- CS_CAN low 
}
//-----------------------------------------------------------------------------
// To avoid a bug, SCK must be low when CS is raising. But thing doesn't work
// if CKP remains low, so we set it to 1 as soon as CS is 1.
//-----------------------------------------------------------------------------
void SPI_mcp_unselect (void)
{
     SBI( CAN_PORT_CS, CS_CAN ); //- CS_CAN high 
     
     SPCR = RettConf;            //- Config SPI zurueck
}
